@@ -994,11 +994,13 @@ create_frmConnect (void)
   gtk_container_set_border_width (GTK_CONTAINER (hbxFileOps), 0);
   gtk_box_set_spacing (GTK_BOX (hbxFileOps), 6);
 
-  btnOpen = gtk_button_new_from_stock ("gtk-open");
+  //btnOpen = gtk_button_new_from_stock ("gtk-open");
+  btnOpen = gtk_button_new_from_stock (GTK_STOCK_OPEN);
   gtk_widget_set_name (btnOpen, "btnOpen");
   gtk_box_pack_start (GTK_BOX (hbxFileOps), btnOpen, TRUE, TRUE, 0);
 
-  btnSaveAs = gtk_button_new_from_stock ("gtk-save-as");
+  //btnSaveAs = gtk_button_new_from_stock ("gtk-save-as");
+  btnSaveAs = gtk_button_new_from_stock (GTK_STOCK_SAVE_AS);
   gtk_widget_set_name (btnSaveAs, "btnSaveAs");
   gtk_box_pack_start (GTK_BOX (hbxFileOps), btnSaveAs, TRUE, TRUE, 0);
 
@@ -1014,11 +1016,13 @@ create_frmConnect (void)
   gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbbAppOps), 6);
   gtk_container_set_border_width (GTK_CONTAINER (hbbAppOps), 6);
 
-  btnHelp = gtk_button_new_from_stock ("gtk-help");
+  //btnHelp = gtk_button_new_from_stock ("gtk-help");
+  btnHelp = gtk_button_new_from_stock (GTK_STOCK_ABOUT);
   gtk_widget_set_name (btnHelp, "btnHelp");
   gtk_container_add (GTK_CONTAINER (hbbAppOps), btnHelp);
 
-  btnQuit = gtk_button_new_from_stock ("gtk-cancel");
+  //btnQuit = gtk_button_new_from_stock ("gtk-close");
+  btnQuit = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
   gtk_widget_set_name (btnQuit, "btnQuit");
   gtk_container_add (GTK_CONTAINER (hbbAppOps), btnQuit);
 
@@ -1280,7 +1284,6 @@ create_frmConnect (void)
 ***************************************/
 
 
-
 void
 on_btnConnect_clicked                  (GtkButton       *button,
                                         gpointer         user_data)
@@ -1288,7 +1291,7 @@ on_btnConnect_clicked                  (GtkButton       *button,
   GtkWidget *main_window;
   rdp_file *rdp = NULL;
   gchar *home = tsc_home_path ();
-  gchar *filename = g_build_path ("/", home, "last.tsc");
+  gchar *filename = g_build_path ("/", home, "last.tsc", NULL);
 
   #ifdef TSCLIENT_DEBUG
   printf ("on_btnConnect_clicked\n");
@@ -1305,9 +1308,17 @@ on_btnConnect_clicked                  (GtkButton       *button,
   gtk_widget_hide (main_window);
   gtk_widget_destroy (main_window);
 
-  if (tsc_launch_remote (rdp, 0) == 0) {
+  gchar** error = g_malloc(sizeof(gchar*));
+
+  if (tsc_launch_remote (rdp, 0, error) == 0) {
     mru_add_server (rdp->full_address);
+  } else {
+    tsc_connect_error (rdp,*error);
   }
+  if(*error)
+      g_free(*error);
+  g_free(error);
+
   // recreate the main window
   create_frmConnect ();
   rdp_file_set_screen (rdp, gConnect);
@@ -1325,7 +1336,7 @@ on_btnQuit_clicked                     (GtkButton       *button,
   GtkWidget *main_window;
   rdp_file *rdp_last = NULL;
   gchar *home = tsc_home_path ();
-  gchar *filename = g_build_path ("/", home, "last.tsc");
+  gchar *filename = g_build_path ("/", home, "last.tsc", NULL);
 
   #ifdef TSCLIENT_DEBUG
   printf ("on_btnQuit_clicked\n");
@@ -1460,7 +1471,7 @@ on_btnSaveAs_clicked                   (GtkButton       *button,
 	  if (lastdelim) {
         GtkWidget *opt = lookup_widget (main_window, "optProfileLauncher");
  	    lastdelim += strlen (home) + 1;
-  	    mi = gtk_menu_item_new_with_mnemonic (lastdelim);
+  	    mi = gtk_menu_item_new_with_label (lastdelim);
         g_signal_connect (G_OBJECT(mi), "activate", G_CALLBACK(tsc_quick_pick_activate), g_strdup (lastdelim));
         gtk_container_add (GTK_CONTAINER(GTK_OPTION_MENU(opt)->menu), mi);
         gtk_widget_show (mi);
