@@ -943,7 +943,7 @@ int rdp_load_profile_launcher (GtkWidget *main_window)
   mnu = gtk_menu_new ();
   while (lptr) {
     name = g_strdup (lptr->data);
-    mi = gtk_menu_item_new_with_mnemonic (name);
+    mi = gtk_menu_item_new_with_label (name);
     g_signal_connect (G_OBJECT (mi), "activate", G_CALLBACK (tsc_quick_pick_activate), name);
     gtk_widget_ref (GTK_WIDGET (mi));
     gtk_container_add (GTK_CONTAINER (mnu), mi);
