@@ -20,11 +20,13 @@ typedef struct
 {
   int alternate_full_switch;
   char *alternate_shell;
+  int attach_to_console;
   int audiomode;
   int auto_connect;
   int bitmapcachepersistenable;
   char *client_hostname;
   int compression;
+  char *description;
   int desktop_size_id;
   int desktopheight;
   int desktopwidth;
@@ -65,3 +67,4 @@ int rdp_file_get_screen (rdp_file *rdp_in, GtkWidget *main_window);
 int rdp_file_set_from_line (rdp_file *rdp_in, const char *str_in);
 int rdp_load_profile_launcher (GtkWidget *main_window);
 int rdp_files_to_list (GSList** list);
+GHashTable* rdp_files_to_hash (void);
