@@ -12,7 +12,6 @@
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <glib.h>
-#include <libgnomeui/gnome-about.h>
 
 #include <libintl.h>
 #define _(x) gettext(x)
@@ -212,7 +211,7 @@ gchar *tsc_home_path () {
  *
  * Evil function to figure out which display number is free.
  *
- * Stolen from gdm-2.6.0:daemon/misc.c
+ * Borrowed from gdm-2.6.0:daemon/misc.c
  *
  * Returns: A free X display number or -1 on failure.
  */
@@ -329,7 +328,7 @@ int tsc_get_free_display (int start, uid_t server_uid)
 *                                      *
 ***************************************/
 
-int tsc_launch_remote (rdp_file *rdp_in, int launch_async)
+int tsc_launch_remote (rdp_file *rdp_in, int launch_async, gchar** error)
 {
   rdp_file *rdp = NULL;
   gchar *c_argv[MAX_ARGVS];
@@ -342,11 +341,15 @@ int tsc_launch_remote (rdp_file *rdp_in, int launch_async)
   gchar *std_out;
   gchar *std_err;
   gint exit_stat = 0;
+  gint retval = 0;
 
   #ifdef TSCLIENT_DEBUG
   printf ("tsc_launch_remote\n");
   #endif
 
+  if(error) {
+    *error=NULL;
+  }
   rdp = rdp_in;
   
   if (strlen(rdp->full_address)) {
@@ -358,7 +361,9 @@ int tsc_launch_remote (rdp_file *rdp_in, int launch_async)
         sflags += G_SPAWN_SEARCH_PATH;
         cmd = "rdesktop";
       } else {
-        tsc_error_message (_("rdesktop was not found in your path.\nPlease verify your rdesktop installation."));
+	if(error) {
+	  *error = g_strdup(_("rdesktop was not found in your path.\nPlease verify your rdesktop installation."));
+	}
         return 1;
       }
       sprintf(buffer, cmd);
@@ -447,7 +452,13 @@ int tsc_launch_remote (rdp_file *rdp_in, int launch_async)
     
       // Extra Options
       if (rdp->audiomode == 0) {
-        sprintf(buffer, "-rsound");
+        sprintf(buffer, "-rsound:local");
+        c_argv[c_argc++] = g_strdup (buffer);
+      } else if (rdp->audiomode == 1) {
+        sprintf(buffer, "-rsound:remote");
+        c_argv[c_argc++] = g_strdup (buffer);
+      } else {
+        sprintf(buffer, "-rsound:off");
         c_argv[c_argc++] = g_strdup (buffer);
       }
 
@@ -518,7 +529,9 @@ int tsc_launch_remote (rdp_file *rdp_in, int launch_async)
       } else if (g_find_program_in_path ("svncviewer")) {
         cmd = "svncviewer";
       } else {
-        tsc_error_message (_("vncviewer or xvncviewer were\n not found in your path.\nPlease verify your vnc installation."));
+	if(error) {
+	  *error = g_strdup(_("vncviewer or xvncviewer were\n not found in your path.\nPlease verify your vnc installation."));
+	}
         return 1;
       }
       sflags += G_SPAWN_SEARCH_PATH;
@@ -527,36 +540,36 @@ int tsc_launch_remote (rdp_file *rdp_in, int launch_async)
       c_argv[c_argc++] = g_strdup (buffer);
 
       if (rdp->screen_mode_id == 2) {
-        sprintf(buffer, "-f");
+        sprintf(buffer, "-FullScreen");
         c_argv[c_argc++] = g_strdup (buffer);
       } else {
         switch (rdp->desktopwidth) {
         case 640:
-          sprintf(buffer, "-g");
+          sprintf(buffer, "-geometry");
           c_argv[c_argc++] = g_strdup (buffer);
           sprintf(buffer, "640x480");
           c_argv[c_argc++] = g_strdup (buffer);
           break;
         case 800:
-          sprintf(buffer, "-g");
+          sprintf(buffer, "-geometry");
           c_argv[c_argc++] = g_strdup (buffer);
           sprintf(buffer, "800x600");
           c_argv[c_argc++] = g_strdup (buffer);
           break;
         case 1024:
-          sprintf(buffer, "-g");
+          sprintf(buffer, "-geometry");
           c_argv[c_argc++] = g_strdup (buffer);
           sprintf(buffer, "1024x768");
           c_argv[c_argc++] = g_strdup (buffer);
           break;
         case 1152:
-          sprintf(buffer, "-g");
+          sprintf(buffer, "-geometry");
           c_argv[c_argc++] = g_strdup (buffer);
           sprintf(buffer, "1152x864");
           c_argv[c_argc++] = g_strdup (buffer);
           break;
         case 1280:
-          sprintf(buffer, "-g");
+          sprintf(buffer, "-geometry");
           c_argv[c_argc++] = g_strdup (buffer);
           sprintf(buffer, "1280x960");
           c_argv[c_argc++] = g_strdup (buffer);
@@ -625,14 +638,18 @@ int tsc_launch_remote (rdp_file *rdp_in, int launch_async)
         sprintf(buffer, "Xnest");
         c_argv[c_argc++] = strdup(buffer);
       } else {
-        tsc_error_message (_("Xnest was not found in your path.\nPlease verify your Xnest installation."));
+	if(error) {
+	  *error = g_strdup(_("Xnest was not found in your path.\nPlease verify your Xnest installation."));
+	}
         return 1;
       }
         
       /* Starting search from :1 (assuming we run at :0) */
       display = tsc_get_free_display (1, getuid());
       if (-1 == display) {
-        tsc_error_message (_("Could not find a free X display."));
+	if(error) {
+	  *error = g_strdup(_("Could not find a free X display."));
+	}
         return 1;
       }
       sprintf(buffer, ":%d", display);
@@ -690,7 +707,9 @@ int tsc_launch_remote (rdp_file *rdp_in, int launch_async)
       } else if (g_file_test ("/usr/lib/ICAClient/wfica", G_FILE_TEST_EXISTS)) {
         cmd = "wfica";
       } else {
-        tsc_error_message (_("wfica was not found in your path.\nPlease verify your ICAClient installation."));
+	if(error) {
+	  *error = g_strdup(_("wfica was not found in your path.\nPlease verify your ICAClient installation."));
+	}
         return 1;
       }
       sprintf(buffer, cmd);
@@ -812,41 +831,134 @@ int tsc_launch_remote (rdp_file *rdp_in, int launch_async)
 
     if (launch_async == 0) {
       if (!g_spawn_sync (NULL, (gchar**)c_argv, NULL, G_SPAWN_SEARCH_PATH,
-            NULL, NULL, &std_out, &std_err, &exit_stat, &err)) {
-        g_warning ("failed: spawn_sync of %s\n", cmd);
-        tsc_error_message ( _("Failed to spawn.\nPlease verify your installation."));
+			 NULL, NULL, &std_out, &std_err, &exit_stat, &err)) {
+	g_warning ("failed: spawn_sync of %s\n", cmd);
+	if(error) {
+	  *error = g_strdup(_("Failed to spawn.\nPlease verify your installation."));
+	}
       }
       if (std_err && strlen(std_err)) {
-        g_warning ("\n%s\n", std_err);
-        tsc_error_message ((gchar*)std_err);
+	g_warning ("\n%s\n", std_err);
+	if(error) {
+	  *error = g_strdup((gchar*)std_err);
+	}
+	retval = 1;
       }
     } else {
       if (!g_spawn_async (NULL, (gchar**)c_argv, NULL, G_SPAWN_SEARCH_PATH,
-            NULL, NULL, NULL, &err)) {
-        g_warning ("failed: spawn_async of %s\n", cmd);
-      }
+			  NULL, NULL, NULL, &err)) 
+	{
+	  g_warning ("failed: spawn_async of %s\n", cmd);
+	}
     }
     if (err) {
       g_warning ("message %s\n", err->message);
-      tsc_error_message (err->message);
+      if(error) {
+	*error = g_strdup( err->message);
+      }
+      retval = 1;
     }
 
     for (cnt = 0; cnt < c_argc; cnt++) {
-      #ifdef TSCLIENT_DEBUG
+#ifdef TSCLIENT_DEBUG
       printf ("arg %d:  %s\n", cnt, c_argv[cnt]);
-      #endif
+#endif
       free (c_argv[cnt]);
     }
-
+    return retval;
   } else {
     // clean up and exit
     return 1;  
   }
   // clean up and exit
-  return 0;
+  return retval;
 }
 
 
+static gboolean 
+tsc_connect_error_timeout(GtkWidget* dialog)
+{
+  gint time = 0;
+  GtkLabel* label = GTK_LABEL(g_object_get_data ((GObject*)dialog, "lblTimeout"));
+  time = (gint)g_object_get_data((GObject*)label, "timeout");
+  if (time) {
+    gchar txt[30];
+    g_sprintf(txt, "reconnect in %d sec", (gpointer)time);
+    g_object_set_data((GObject*)label, "timeout", (gpointer)time-1);
+    gtk_label_set_text(label, txt);
+  } else {
+    gtk_dialog_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
+  }
+  return TRUE;
+}
+
+void tsc_connect_error (rdp_file * rdp, const gchar* error)
+{
+  GtkWidget *dialog, *err_label, *label, *label_detail, *image, *hbox, *vbox, *expander;
+  gint responce, c_id;
+  
+#ifdef TSCLIENT_DEBUG
+  printf ("tsc_error_message\n");
+#endif
+  
+  if (!error)
+    error = "";
+
+  /* create the widgets */
+  dialog = gtk_dialog_new_with_buttons ( _("Terminal Server Client Error"),
+					 NULL,
+					 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+					 GTK_STOCK_CONNECT,
+					 GTK_RESPONSE_OK,
+					 GTK_STOCK_CANCEL,
+					 GTK_RESPONSE_CANCEL,
+					 NULL);
+
+  err_label = gtk_label_new ( _("An error has occurred."));
+  label     = gtk_label_new ("");
+  image     = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING, GTK_ICON_SIZE_DIALOG);
+  hbox      = gtk_hbox_new (FALSE, 12);
+  vbox      = gtk_vbox_new (FALSE, 0);
+
+  gtk_label_set_markup (GTK_LABEL (err_label), g_strconcat ("<span weight=\"bold\">", _("An error has occurred."), "</span>", NULL));
+  gtk_label_set_justify (GTK_LABEL (err_label), GTK_JUSTIFY_LEFT);
+  g_object_set_data((GObject*)dialog, "lblTimeout", label);
+
+  /* add the label, and show everything we've added to the dialog. */
+  gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 12);
+  gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 12);
+  gtk_box_pack_start (GTK_BOX (vbox), err_label, TRUE, TRUE, 12);
+  gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 12);
+  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), hbox);
+  gtk_box_set_spacing ((GtkBox*)GTK_DIALOG(dialog)->vbox, 12);
+  
+  /* Create the expander */
+  expander = gtk_expander_new (_("Details"));
+  gtk_box_pack_start (GTK_BOX((GTK_DIALOG(dialog)->vbox)), expander, FALSE, FALSE, 0);
+  label_detail = gtk_label_new (error);
+  gtk_container_add (GTK_CONTAINER (expander), label_detail);
+
+  gtk_widget_show_all(dialog);
+
+  g_object_set_data((GObject*)label, "timeout", (gpointer)30);
+  tsc_connect_error_timeout(dialog);
+  c_id = gtk_timeout_add(1000,(GtkFunction)tsc_connect_error_timeout,dialog) ;
+
+  responce = gtk_dialog_run (GTK_DIALOG(dialog));
+  g_source_remove(c_id);
+
+  gtk_widget_destroy(dialog);
+  if(responce == GTK_RESPONSE_OK) {
+    gchar** err2 = g_malloc(sizeof(gchar*));
+    if (tsc_launch_remote ( rdp, 0, err2) != 0) {
+	tsc_connect_error(rdp, *err2);
+    }
+    if(*err2)
+      g_free(*err2);
+    g_free(err2);
+  }
+}
+
 void tsc_error_message (gchar *message)
 {
   GtkWidget *dialog, *err_label, *label, *image, *hbox, *vbox;
@@ -885,7 +997,8 @@ void tsc_error_message (gchar *message)
   gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 12);
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), hbox);
   gtk_box_set_spacing ((GtkBox*)GTK_DIALOG(dialog)->vbox, 12);
-  gtk_widget_show_all (dialog);
+  gtk_widget_show_all(dialog);
+  gtk_dialog_run (GTK_DIALOG(dialog));
 }
 
 
@@ -894,34 +1007,36 @@ void tsc_about_dialog ()
   static GtkWidget *about = NULL;
   const gchar *name      = _("Terminal Server Client");
   const gchar *version   = VERSION;
-  const gchar *copyright = "(C) 2003 Erick Woods\n<http://www.gnomepro.com/tsclient/>";
+  const gchar *copyright = "(C) 2002-2006 Erick Woods";
+  const gchar *website = "http://www.gnomepro.com/tsclient/";
   const gchar *authors[] = { "Erick Woods <erick@gnomepro.com>", "Kyle Davis <kyle@gnomeuser.org>", NULL };
-  const gchar *documenters[] = { NULL };
-  const gchar *translator_credits = NULL;
-  GdkPixbuf *pixbuf = create_pixbuf ("tsclient.png");
+  const gchar *artists[] = { "Erick Woods <erick@gnomepro.com>", "Jakub Steiner", NULL };
+  //const gchar *documenters[] = { NULL };
+  //const gchar *translator_credits = NULL;
+  GdkPixbuf *logo = create_pixbuf ("tsclient.png");
+
+  const gchar *license = \
+    "Terminal Server Client is licensed under the \n" \
+    "GNU General Public License (GPL)\n<http://www.gnu.org/licenses/gpl.html>";
 
   const gchar *comments = \
-    "Terminal Server Client is a frontend for \nrdesktop <http://www.rdesktop.org/> \nand vncviewer.\n" \
-    "\n" \
-    "Terminal Server Client is licensed under the \n" \
-    "GNU General Public License (GPL)\n<http://www.gnu.org/licenses/gpl.html>\n" \
-    "\n" \
-    "Images by: Jakub \"jimmac\" Steiner and others.";
+    "Terminal Server Client is a frontend for \nrdesktop, vncviewer, wfica and xnest.";
   
   if (about) {
     gtk_window_present (GTK_WINDOW (about));
     return;
   }
   
-  about = gnome_about_new ( 
-        name,
-        version,
-        copyright,
-        comments, 
-        authors, 
-        documenters, 
-        translator_credits,  
-        pixbuf);
+  about = gtk_about_dialog_new ();
+  gtk_about_dialog_set_name (GTK_ABOUT_DIALOG(about), name);
+  gtk_about_dialog_set_version (GTK_ABOUT_DIALOG(about), version);
+  gtk_about_dialog_set_copyright (GTK_ABOUT_DIALOG(about), copyright);
+  gtk_about_dialog_set_comments (GTK_ABOUT_DIALOG(about), comments);
+  gtk_about_dialog_set_website (GTK_ABOUT_DIALOG(about), website);
+  gtk_about_dialog_set_authors (GTK_ABOUT_DIALOG(about), authors);
+  gtk_about_dialog_set_license (GTK_ABOUT_DIALOG(about), license);
+  gtk_about_dialog_set_artists (GTK_ABOUT_DIALOG(about), artists);
+  gtk_about_dialog_set_logo (GTK_ABOUT_DIALOG(about), logo);
   
   //gtk_window_set_wmclass (GTK_WINDOW (about), "terminal server client", "Terminal Server Client");
   g_signal_connect (G_OBJECT (about), "destroy", G_CALLBACK (gtk_widget_destroyed), &about);
@@ -948,9 +1063,14 @@ void tsc_quick_pick_activate (GtkMenuItem *menuitem, gpointer user_data)
     rdp = g_new0 (rdp_file, 1);
     rdp_file_init (rdp);
     rdp_file_load (rdp, file_name);
-    if (tsc_launch_remote (rdp, 1) == 0) {
+    
+    gchar** error = g_malloc(sizeof(gchar*));
+    if (tsc_launch_remote (rdp, 1,error) == 0) {
       //mru_add_server (rdp->full_address);
     }
+    if(*error)
+      g_free(error);
+    g_free(error);
   }
   g_free (rdp);
   g_free (file_name);
